<?php

namespace PHPImageWorkshop;

use PHPImageWorkshop\Core\ImageWorkshopLayer as ImageWorkshopLayer;
use PHPImageWorkshop\Core\ImageWorkshopLib as ImageWorkshopLib;
use PHPImageWorkshop\Exception\ImageWorkshopException as ImageWorkshopException;

// If no autoloader, uncomment these lines:
require_once(__DIR__.'/Core/ImageWorkshopLayer.php');
require_once(__DIR__.'/Exception/ImageWorkshopException.php');
class ImageWorkshop
{
    const ERROR_NOT_AN_IMAGE_FILE = 1;
    
    const ERROR_IMAGE_NOT_FOUND = 2;
    
    const ERROR_NOT_WRITABLE_FILE = 3;
    
    const ERROR_CREATE_IMAGE_FROM_STRING = 4;
    
    public static function initFromPath($path)
    {
        if (file_exists($path) && !is_dir($path)) {
            
            if (!is_readable($path)) {
                throw new ImageWorkshopException('Can\'t open the file at "'.$path.'" : file is not writable, did you check permissions (755 / 777) ?', static::ERROR_NOT_WRITABLE_FILE);
            }
            
            $imageSizeInfos = @getImageSize($path);
            $mimeContentType = explode('/', $imageSizeInfos['mime']);
            
            if (!$mimeContentType || !array_key_exists(1, $mimeContentType)) {
                throw new ImageWorkshopException('Not an image file (jpeg/png/gif) at "'.$path.'"', static::ERROR_NOT_AN_IMAGE_FILE);
            }
            
            $mimeContentType = $mimeContentType[1];
            
            switch ($mimeContentType) {
                case 'jpeg':
                    $image = imageCreateFromJPEG($path);
                break;

                case 'gif':
                    $image = imageCreateFromGIF($path);
                break;

                case 'png':
                    $image = imageCreateFromPNG($path);
                break;

                default:
                    throw new ImageWorkshopException('Not an image file (jpeg/png/gif) at "'.$path.'"', static::ERROR_NOT_AN_IMAGE_FILE);
                break;
            }
            
            return new ImageWorkshopLayer($image);
        }
        
        throw new ImageWorkshopException('No such file found at "'.$path.'"', static::ERROR_IMAGE_NOT_FOUND);
    }
    
    public static function initTextLayer($text, $fontPath, $fontSize = 13, $fontColor = 'ffffff', $textRotation = 0, $backgroundColor = null)
    {
        $textDimensions = ImageWorkshopLib::getTextBoxDimension($fontSize, $textRotation, $fontPath, $text);

        $layer = static::initVirginLayer($textDimensions['width'], $textDimensions['height'], $backgroundColor);
        $layer->write($text, $fontPath, $fontSize, $fontColor, $textDimensions['left'], $textDimensions['top'], $textRotation);
        
        return $layer;
    }
    
    public static function initVirginLayer($width = 100, $height = 100, $backgroundColor = null)
    {
        $opacity = 0;
        
        if (!$backgroundColor || $backgroundColor == 'transparent') {
            $opacity = 127;
            $backgroundColor = 'ffffff';
        }
        
        return new ImageWorkshopLayer(ImageWorkshopLib::generateImage($width, $height, $backgroundColor, $opacity));
    }
    
    public static function initFromResourceVar($image)
    {
        return new ImageWorkshopLayer($image);
    }
    
    public static function initFromString($imageString)
    {
        if (!$image = @imageCreateFromString($imageString)) {
            throw new ImageWorkshopException('Can\'t generate an image from the given string.', static::ERROR_CREATE_IMAGE_FROM_STRING);
        }
        
        return new ImageWorkshopLayer($image);
    }
}